<?php

/**
 * @link              https://alexrevo.pw/
 * @since             0.0.1
 * @package           Revo_Debug
 *
 * @wordpress-plugin
 * Plugin Name:       Revo Debug
 * Plugin URI:        https://alexrevo.pw/product/revo-debug-plugin.html
 * Description:       Yootheme Pro bedug tests and tools
 * Version:           0.0.1
 * Release date:      10.06.2022
 * Author:            Alex Revo
 * Author URI:        https://alexrevo.pw/
 */

// (На основе wp522/yootheme-code-import, в т.ч. с текстовым полем)
// (На основе ao-l*buket-settings 0.0.5)

// Exit if accessed directly
defined('ABSPATH') || die('');

//Author Andrey Ovdienko
class AoYoothemeApikeysSet
{

    const ADMIN_MENU_NAME = "Revo Debug";
    const ADMIN_MENU_SLUG = "revo_debug";
    const ADMIN_MENU_POSITION = 66;
    const ADMIN_MENU_ICON = '';


    private static $_self = null;

    private function __construct()
    {
    }
    private function __clone()
    {
    }
    private function __wakeup()
    {
    }


    public static function init()
    {
        if (self::$_self === null) {
            self::$_self = new self();
            self::$_self->_init();
        }
        return self::$_self;
    }


    private function _init()
    {
        add_action('admin_menu', array($this, 'admin_menu_item'));
        add_action('admin_init', array($this, 'register_settings'));
        add_action('admin_init', array($this, 'add_settings_section'));

        $this->_options = $this->get_themes_mod_options_list();
    }


    /* Страница настроек */

    public function admin_menu_item()
    {
        $page_title = self::ADMIN_MENU_NAME; // title
        $menu_title = self::ADMIN_MENU_NAME; // пункт меню
        $capability = 'manage_options';
        $menu_slug  = self::ADMIN_MENU_SLUG;
        $function   = array($this, 'admin_menu_page');
        $icon_url   = self::ADMIN_MENU_ICON;
        $position   = self::ADMIN_MENU_POSITION;

        add_menu_page($page_title, $menu_title, $capability, $menu_slug, $function, $icon_url, $position);
    }

    public function admin_menu_page()
    {
        include 'tpl/admin_menu_page.php';
    }




    private $_options = [];


    private static function buildThemeOptionTitle($theme)
    {
        $format = '%s <span style="font-weight: normal;">%s<br><small><i>%s</i> %s</small></span>';
        $title = sprintf($format, $theme['name'], $theme['version'], $theme['stylesheet'], $theme['is_current'] ? '<b style="background: #1d2327; color: #f0f0f1; padding: 0 5px 1px; border-radius: 3px; font-size: 90%; margin-left: 10px;">Active</b>' : '');
        return $title;
    }


    private function get_themes_mod_options_list()
    {
        $mods_list = [];
        $yoothemes = $this->get_yoothemes_list();
        foreach ($yoothemes as $theme) {
            $mods_list[$theme['option']] = [
                "title" => self::buildThemeOptionTitle($theme),
                "sanitize_callback" => function ($val) use ($theme) {

                    $theme_slug = $theme['stylesheet'];

                    $option = $this->build_theme_mods_option_name($theme_slug);
                    $mods = get_option($option);
                    $config = json_decode($mods['config'] ?? '{}', true);

                    $config['yootheme_apikey'] = $val;

                    $mods['config'] = json_encode($config);

                    return $mods;
                },
                "value" => $theme["yootheme_apikey"]
            ];
        }
        return $mods_list;
    }


    private function get_yoothemes_list()
    {

        global $wp_theme_directories;

        // var_dump($wp_theme_directories);

        $current_theme = wp_get_theme();
        $current_theme_stylesheet = $current_theme->get_stylesheet();
        $current_theme_root = $current_theme->get_theme_root();

        $yoothemes_list = [];

        if (!empty($wp_theme_directories)) foreach ($wp_theme_directories as $theme_root) {
            if (!file_exists($theme_root) || !is_dir($theme_root)) {
                continue;
            }
            $scan = scandir($theme_root);

            foreach ($scan as $theme_dir) {
                if ($theme_dir == '.' || $theme_dir == '..' || !is_dir($theme_root . DIRECTORY_SEPARATOR . $theme_dir)) {
                    continue;
                }
                $theme = wp_get_theme($theme_dir, $theme_root);
                if ($theme->errors()) {
                    continue;
                }
                $parent = $theme->parent();

                if ($theme->get_stylesheet() != 'yootheme') {
                    if (!$parent || $parent->get_stylesheet() != 'yootheme') {
                        continue;
                    }
                }

                $item = [
                    'stylesheet' => $theme->get_stylesheet(),
                    'theme_root' => $theme_root,
                    'name'       => $theme->get('Name'),
                    'version'    => $theme->get('Version'),
                    'parent'     => false,
                    'is_current' => false
                ];

                if ($item['stylesheet'] == $current_theme_stylesheet && $item['theme_root'] == $current_theme_root) {
                    $item['is_current'] = true;
                }

                if ($parent) {
                    $item['parent'] = [
                        'stylesheet' => $parent->get_stylesheet(),
                        'name'       => $parent->get('Name'),
                        'version'    => $parent->get('Version'),
                    ];
                }

                $theme_slug = $item['stylesheet'];

                $item['option'] = $this->build_theme_mods_option_name($theme_slug);

                $config = $this->get_theme_config($theme_slug);
                $item['yootheme_apikey'] = $config['yootheme_apikey'] ?? '';

                $yoothemes_list[] = $item;
            }
        }

        return $yoothemes_list;
    }

    private function build_theme_mods_option_name($theme_slug)
    {
        return "theme_mods_{$theme_slug}";
    }

    private function get_theme_config($theme_slug)
    {
        $option = $this->build_theme_mods_option_name($theme_slug);
        $mods = get_option($option);
        $config = json_decode($mods['config'] ?? '{}', true);
        return $config;
    }


    public function register_settings()
    {
        foreach ($this->_options as $option_key => $option_title) {

            $option_settings = [];

            if (is_object($option_title)) {
                $option_title = (array) $option_title;
            }

            if (is_array($option_title)) {
                $option_settings = $option_title;
                $option_title = $option_settings['title'];
            }

            if (is_int($option_key)) {
                $option_key = $option_title;
            }

            $this->register_setting($option_key, $option_settings);

            if (!isset($option_settings['add_field']) || $option_settings['add_field'] == true) {
                $this->add_settings_field($option_key, $option_title, $option_settings);
            }
        }
    }



    public function register_setting($option_key, $option_settings = [])
    {

        $option_group = self::ADMIN_MENU_SLUG;
        $option_name  = $option_key;

        $sanitize_callback = null;

        if (!empty($option_settings['sanitize_callback_this'])) {
            $sanitize_callback = array($this, $option_settings['sanitize_callback_this']); // 0.0.5
        } elseif (!empty($option_settings['sanitize_callback'])) {
            $sanitize_callback = $option_settings['sanitize_callback']; // 0.0.5
        }



        $args = array(
            'type'              => 'string', // Only used by the REST API
            'group'             => $option_group,
            'description'       => '', // Only used by the REST API
            'sanitize_callback' => $sanitize_callback,
            'show_in_rest'      => false,
        );

        // Регистрирует новую опцию и callback функцию для обработки значения опции при её сохранении в БД.

        register_setting($option_group, $option_name, $args);
    }



    public function add_settings_section()
    {

        $id       = 'default';
        $title    = '';
        $callback = array($this, 'print_section_description');
        $page     = self::ADMIN_MENU_SLUG;

        add_settings_section($id, $title, $callback, $page);
    }



    public function print_section_description()
    {
?>
        <?php
    }



    public function add_settings_field($option_key, $option_title = null, $option_settings = [])
    {

        if (!$option_title) {
            $option_title = $option_key;
        }

        $type = isset($option_settings['type']) ? $option_settings['type'] : 'input';

        $id       = $option_key;
        $title    = $option_title;
        $callback = array($this, "print_input");
        $page     = self::ADMIN_MENU_SLUG;
        $section  = 'default';

        $args = array(
            /* Передаются в callback */
            'id'          => $option_key,
            'option_name' => $option_key,
            'tpl'         => $type
        ) + $option_settings;

        add_settings_field($id, $title, $callback, $page, $section, $args);
    }



    public function print_input($val)
    {
        $id          = $val['id'];
        $option_name = $val['option_name'];
        $tpl         = $val['tpl'];
        $value       = esc_attr($val['value'] ?? get_option($option_name));

        $path = dirname(__FILE__) . DIRECTORY_SEPARATOR . "tpl/{$tpl}.php";

        if (file_exists($path)) {
            include $path;
        } else {
            echo "No tpl {$tpl}";
        }
    }
}


AoYoothemeApikeysSet::init();
